//    case 118: //Текущее состояние - фракционная ректификация
//    case 109: //Текущее состояние - ректификация

// Функция возвращает значение ШИМ для отборв по "шпоре" (температуре в кубе)
char GetCHIMOtbor()
{
  // Процент отбора задается в табличном виде, например в данном случае 
  // от 0 до 88 в кубе процент отбора не меняем, от 88.0 до 96.0 в кубе процент отбора ставить в промежутке от 68 до 25
  // от 960 до 1000 процент отбора понижаем до нуля.
  // здесь храним количество значений в таблице;
  char FindCHIM;
  char i;
  #ifdef TEST
    Serial.print("KUB=");
    Serial.print(temps[TEMP_KUB]);
  #endif
  
  FindCHIM=ProcChimSR;
  for(i=1;i<CntCHIM;i++)
  {
    if (tempK[i-1]<=temps[TEMP_KUB] && tempK[i]>temps[TEMP_KUB])
    {
      // Если отбор при этом диапазоне температур производим
      if (CHIM[i-1]>0) 
      {
        FindCHIM=(temps[TEMP_KUB]-tempK[i-1])*(int) (CHIM[i]-CHIM[i-1])/(tempK[i]-tempK[i-1])+(int) CHIM[i-1];
        #ifdef TEST
          Serial.print(" CHIM=");
          Serial.println((int)FindCHIM);
        #endif
        return FindCHIM; // Теперь возвращаем новое значение ШИМ в любом случае. 
      }
      
    }
  }
  
  #ifdef TEST
  Serial.println(" noFindCHIM");
  #endif

  return ProcChimSR; 
}

void ProcessRectif()
{

  //#define KLP_NPG  0      // Номер клапана для управления НПГ (при дистилляции) 
  //#define KLP_VODA 1      // Номер клапана для управления общей подачей воды в систему
  //#define KLP_DEFL 2      // Номер клапана для подачи воды в дейфлегматор
  //#define KLP_GLV_HVS  3       // Номер клапана отбора головных и хвостовых фракций
  //#define KLP_SR   4      // Номер клапана отбора ректификата
  //#define TEMP_KUB  0      // Номер датчика  термометра в кубе
  //#define TEMP_RK20 1      // Номер датчика термометра в РК 20 см от насадки
  //#define TEMP_TSA  2      // Номер термометра в трубке связи с атмосферой
  static char Count_Provodimost=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  static char OldStateMachine=0; // Количество срабатываний датчика проводимости для окончания отбора голов.
  int V3; // Текущий уровень проводимости;

  if (flAlarmUroven && StateMachine<8 ) 
  { // Переводим автомат в стадию завершения 
    
    if (IspReg!=118)
    {  // Не фракционная перегонка, все как обычно, останавливаем процесс.
      if (USE_ALARM_UROVEN==1)
        {
          StateMachine=8; // Переводим в режим окончания
          SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
          if (BeepStateProcess) my_beep(BEEP_LONG);
        }
        else
        {
          OldStateMachine=StateMachine;
          StateMachine=9;  // Переводим в режим ожидания
          if (BeepStateProcess) my_beep(BEEP_LONG);
          Count_Provodimost=0;
          KlOpen[KLP_GLV_HVS]=0;
          KlClose[KLP_GLV_HVS]=100;
          KlOpen[KLP_SR]=0;
          KlClose[KLP_SR]=100;
        }
   }
   else
   {
    // Фракционная ректификация - переходим на следующую фракцию
        TekFraction++; SecTempPrev1=Seconds;
       flAlarmUroven=0; 
       CountAlarmUroven=0;
       if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
   }   
  }

  // Проверяем температуру в ТСА
  if (temps[TEMP_TSA]>=MAX_TEMP_TSA) 
  {
    StateMachine=101; // Переводим автомат в состояние аварии по ТСА.
  }
  if (flAlarmMPX5010) StateMachine=102; // Переводим в режим тревоги по датчику давления

  // Проверяем окончание температуры окончания ректификации
  // Окончание ректификации также определяется, если шим меньше нуля, для этого минимальный шим  надо поставить 0
  if ((temps[TEMP_KUB]>=tEndRect)  && StateMachine<8 && IspReg!=118 )
  {  
    StateMachine=8;        
    SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
    if (BeepStateProcess) my_beep(BEEP_LONG);
  }

  switch  (StateMachine)
  {
  case 0: // Процесс не запущен
    break;
  case 1: // Начало процесса
    PrepareProcess();
    KlReg[KLP_GLV_HVS]=PEREGREV_ON;
    KlReg[KLP_SR]=PEREGREV_ON;

    //          if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();
    lastSMSState=0;
    KlOpen[KLP_VODA]=PER_KLP_OPEN;
    KlClose[KLP_VODA]=PER_KLP_CLOSE;
//    digitalWrite(PIN_REG_ON,HIGH);
    digitalWrite(PIN_RZG_ON,RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,!ALL_OFF_HIGH);
    digitalWrite(PIN_START_1,RELAY_HIGH);
    digitalWrite(PIN_START_2,RELAY_HIGH);
    
    flAllOff=0;
    StateMachine=2;
    if (BeepStateProcess) my_beep(BEEP_LONG);
    Count_Provodimost=0;
    if (CntCHIM<0) CntCHIM=-1;
    
  case 2: // Разгон
    UstPower=Power; // Устанавливаем максимальную мощность для разгона
    // Подчиненные контроллеры переводим в разгон
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    // Учитываем как температуру в кубе, так и температуру в колонне, в зависимости от того, положительные или отрицательные значения температур.
    if (temps[TEMP_KUB]<tEndRectRazgon  && tEndRectRazgon>0) break;
    if (temps[TEMP_RK20]<-tEndRectRazgon && tEndRectRazgon<=0) break;

    // Подаем питание на клапан дефлегматора
    KlOpen[KLP_DEFL]=PER_KLP_OPEN;
    KlClose[KLP_DEFL]=PER_KLP_CLOSE;
    TempPrev=temps[TEMP_RK20];
    SecTempPrev=Seconds; // Запоминаем дату последнего изменения температуры
    StateMachine=3;        
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    if (BeepStateProcess) my_beep(BEEP_LONG);
    TekFraction=0;
    SetAngle(AngleFractionRect[TekFraction]);
    SecTempPrev=Seconds;
  case 3:// Стабилизация температуры
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    // Польский буфер откроем.
    KlOpen[KLP_PB]=200;
    KlClose[KLP_PB]=0;
    
    if (temps[TEMP_RK20]>0) // Если температура больше 70 градусов (вдруг куб прогрелся, а колонна еще нет, нелогично, конечно, но на всякий случай учтем)
    {
      if (TimeStabKolonna>=0)  // Если время стабилизации колонны больше нуля, то тогда считаем время относительно последнего изменения температуры
      {
        if (abs(temps[TEMP_RK20]-TempPrev)<2) //Если текущая температура колонны равна температуре, запомненной ранее за исключением погрешности 0,2 градуса
        {
          if (Seconds-SecTempPrev>TimeStabKolonna) //Если с момента последнего измерения прошло более 600 секунд (10 минут), тогда считаем, что температура в колоенне стабилизировалась
          {
            // Переходим к следующему этапу - отбору голов. 
            StateMachine=4;        
            if (BeepStateProcess) my_beep(BEEP_LONG);
            SecTempPrev=Seconds;
          }
          else
            break;
  
        }
        else
        {
          TempPrev=temps[TEMP_RK20];// Запоминаем температуру
          SecTempPrev=Seconds; // Запоминаем время последнего изменения температуры
          break;
        }
      }
      else
      {
        // Если время стабилизации колонны меньше нуля, то тогда считаем абсолютное время 
        if (Seconds-SecTempPrev>-TimeStabKolonna) //Если с момента начала стабилизации прошло больше заданного количества секунд
          {
            // Переходим к следующему этапу - отбору голов. 
            StateMachine=4;        
            if (BeepStateProcess) my_beep(BEEP_LONG);
            SecTempPrev=Seconds;
          }
          else
            break;
      }
    }
    else
      break;
  case 4: // Отбор голов
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    //unsigned int timeChimRectOtbGlv=2000;  // Шим отбора голов (в полупериодах сетевого напряжения в данном случае это около 20 секунд)
    //unsigned char ProcChimOtbGlv=5;  // Процент ШИМ отбора голов (в данном случае это 5 процентов)
    // Клапан отбора ректификата закроем на всякий случай
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;
    
    // Устанавливаем клапан отбора хвостов и голов в соответвии с установленнным ШИМ
    //int tEndRectOtbGlv=854;     // Температура окончания отбора голов 85.4 С
    if (ProcChimOtbGlv>0)
        KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
    else
        KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
            
    KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
    
    if (IspReg!=118)
    { // Обычная ректификация
      if (UrovenProvodimostSR==0) // Если проводимость спирта не используется, головы отбираем по температуре
      {
        if (temps[TEMP_KUB]<tEndRectOtbGlv)
        {  // Пока температура в кубе не выросла выше окончания отбора голов, продолжаем это состояни
          break;       
        }
      }
      else
      {
  
        if (UrovenProvodimostSR>0) // Если проводимость больше нуля, используем проводимость в том или ином виде
        {
          V3=U_GLV; 
          // Если проводимость менее заданного уровня, и проводимость вообще сущетвует, то увеличиваем счетчик срабатываний
          // или ести проводимость больше уровня тревоги и установлен признак отбирать головы по уровню в приемной емкости.
    
          if ((V3<UrovenProvodimostSR && V3>=2 && UrovenProvodimostSR>=5) || (V3>=UROVEN_ALARM && UrovenProvodimostSR<=2 ))
           { 
             my_beep(BEEP_LONG); // Сначала пищим, предупреждая.
             Count_Provodimost++;
           }
          else 
            Count_Provodimost=0;
    
          // Если счетчик срабатываний не превысил максимальное число раз, тогда продолжаем процесс отбора голов.
          if (Count_Provodimost<MAX_COUNT_PROVODIMOST_SR) break;
        } 
        else // Если уровень проводимости SR меньше нуля, тогда это означает, что отбор голов производится по времени (в десятках минут).
        {
          // Анализируем, сколько минут прошло с начала отбора голов.
          SecOstatok=-((Seconds-SecTempPrev) / 600 + UrovenProvodimostSR);
          if (SecOstatok>0) break;
        }
      }
    }
    else
    {
      // При фракционном отборе отбор продукта идет через клапан СР.
      KlOpen[KLP_SR]=KlOpen[KLP_GLV_HVS];
      KlClose[KLP_SR]=KlClose[KLP_GLV_HVS];

      if (TekFraction==0) // Текущая фракция при перегонке всегда равна нулю, если вдруг она сменилась, например по наполнению приемной емкости (или вручную ее поменяли), тогда перейдем на отбор СР
      {
        // Фракционная ректификация - там немного по-другому
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
          else 
            break;
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
             
           }
           else 
             break;
        }
      }    
    }
    // Если температура превысила указанный рубеж, то переходим в состояние отбора спирта-ректификата.
    // Клапан отбора голов и хвостов отключаем
    tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    ProcChimSR=begProcChimOtbSR;
    StateMachine=5;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
    SecTempPrev1=Seconds;// Запомним время, когда произошел переход с фракции голов на отбор тела
    time1=5;
  case 5: // Ожидание, пока температура не вернется к состоянию стабилизации
    // Это для отладки страт-стопа, на самом деле температура 28,5 градуса в этом режиме в колонне быть не может
    if (tStabSR==0) tStabSR=285;
    // Если колонна слишком долго находится в режиме стопа, то температуру стабилизации примем за новую
    if (time1==0 && TimeRestabKolonna!=0) 
     { 
       tStabSR=temps[TEMP_RK20];// Это температура, относительно которой будем стабилизировать отбор
       if (BeepStateProcess) my_beep(BEEP_LONG);
       // Если время рестабилизации колонны меньше нуля, тогда при долгом простое переходим на отбор хвостов.
       if (TimeRestabKolonna<0)
       {
            if (IspReg!=118)// Если не фракционная перегонка, то все как обычно.
            {
               // При не фракционной перегонке
                StateMachine=7;        
                KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
                KlClose[KLP_SR]=10;
                if (BeepStateProcess) my_beep(BEEP_LONG); 
                break;
            }
            else
            {
               // Переходим на последнюю фракцию, то есть отбор хвостов.
               SecTempPrev1=Seconds; 
               TekFraction=CountFractionRect-1;
               if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
               StateMachine=7;    
               break;                 
            }
       }
     }
     
    KlOpen[KLP_GLV_HVS]=0;
    KlClose[KLP_GLV_HVS]=10;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;
    if (temps[TEMP_KUB]>=tEndRectOtbSR)
    {  
      StateMachine=7;        
      if (BeepStateProcess) my_beep(BEEP_LONG);
      break;
    }

    if (temps[TEMP_RK20]>tStabSR)
    {
      break;
    }
    SecTempPrev=Seconds;// Запомним время, когда стабилизировалась температура
    StateMachine=6;        
    if (BeepStateProcess) my_beep(BEEP_LONG);
  case 6: // Отбор СР
    //unsigned int timeChimRectOtbSR=1000;  // Шим отбора СР (в полупериодах сетевого напряжения в данном случае это около 10 секунд)
    //unsigned char tDeltaRect=10;  // Дельта ректификации (в десятых градуса Цельсия) в данном случае - это 10 десятых, то есть 1 градус Цельсия
    //unsigned int tEndRectOtbSR=965;  // Температура окончания отбора спирта-ректификата и начала отбора хвостов (в данном случе 96.5 С)
    // Работа Клапана отбора голов зависит от работы царги пастеризации
    
  if (ProcChimOtbCP>=0)
    {
          if (ProcChimOtbGlv>0)
              KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*ProcChimOtbGlv;
          else
              KlOpen[KLP_GLV_HVS]=-ProcChimOtbGlv;
              
          KlOpen[KLP_GLV_HVS]=((long) KlOpen[KLP_GLV_HVS]*(long)ProcChimOtbCP)/100;
          KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
      
    }
    else
    {
      KlOpen[KLP_GLV_HVS]=-ProcChimOtbCP;
      KlClose[KLP_GLV_HVS]=timeChimRectOtbSR-KlOpen[KLP_GLV_HVS];
    }
    
    // Польский буфер закроем.
    KlOpen[KLP_PB]=0;
    KlClose[KLP_PB]=10;

    // Устанавливаем ШИМ отбора СР
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    ProcChimSR=GetCHIMOtbor();  // Реализуется отбор по-шпоре, что в функции прописано то и будет возвращено.
    if (ProcChimSR>0) KlOpen[KLP_SR]=((timeChimRectOtbSR/10)*ProcChimSR)/10;
                 else KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=timeChimRectOtbSR-KlOpen[KLP_SR];
    
    if (temps[TEMP_RK20]>=tStabSR+tDeltaRect) // Если текущая температура превысила базовую 
    {
      //        //Если температура превысилась менее, чем за одну минуту
      //        if (Seconds-SecTempPrev<60)
      //          {
      // произошел стоп - если значение количества температур в кубе меньше нуля, тогда запомним температуру в кубе за вычетом 0.1 градуса и процент ШИМ, при котором это произошло.
      if (CntCHIM<0)
      {
        // Запоминаем температуру, когда произошел стоп за вычетом 0.1 градуса.
        tempK[-CntCHIM]=temps[TEMP_KUB]-1;
        CHIM[-CntCHIM]=ProcChimSR;
        if (-CntCHIM<COUNT_CHIM-1) CntCHIM--;
      }

      if (DecrementCHIM>=0) ProcChimSR=ProcChimSR-DecrementCHIM;  // Тогда уменьшаем  ШИМ указанное число процентов в абсолютном выражении
         else  
         {
           V3=((int) ProcChimSR* (int)-DecrementCHIM)/100;
           // Процентное отношение может быть очень мало, поэтому если получилось нулевое значение, то вычтем единицу.
           if (V3<=0) V3=1;
           ProcChimSR=ProcChimSR-(char) V3; // Тогда увеличиваем ШИМ на число процентов в относительном выражении
         }

      if (ProcChimSR<minProcChimOtbSR) ProcChimSR=minProcChimOtbSR;

      // Если в результате стопа произошло уменьшение отбора до -1 и минимальный тоже -1, тогда переходим на хвосты.
      if ((ProcChimSR==-1 && minProcChimOtbSR==-1) && IspReg!=118 )
      {  
        StateMachine=7;        
        if (BeepStateProcess) my_beep(BEEP_LONG);
        KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
        KlClose[KLP_SR]=10;
        break;
      }
      
      // Если в результате стопа произошло уменьшение отбора до 0 и минимальный тоже 0, тогда переходим на окончание отбора.
      if ((ProcChimSR==0 && minProcChimOtbSR==0) && IspReg!=118 )
      {  
        StateMachine=8;        
        SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
        if (BeepStateProcess) my_beep(BEEP_LONG);
        break;
      }
  
      
      StateMachine=5;        
      // Взводим таймер для возможной рестабилизации колонны
      if (TimeRestabKolonna<0) time1=-TimeRestabKolonna;
                          else time1=TimeRestabKolonna; 
                          
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    else // Если температура не выросла более, чем за 10 минут, прибавим ШИМ на 5%
    {
      if (Seconds-SecTempPrev>TimeAutoIncCHIM) // Если не стабилизировалось более, чем за 10 минут, тогда прибавляем ШИМ на 5%
      {

        // Шим прибавляем только если не дошли до минимального шима, если дошли, то считаем, что хватит играть ШИМом
        if (ProcChimSR>minProcChimOtbSR) 
        {
          if (IncrementCHIM>=0)   ProcChimSR=ProcChimSR+IncrementCHIM; // Тогда увеличиваем ШИМ указанное число процентов в абсолютном выражении
                          else    ProcChimSR=ProcChimSR+((int) ProcChimSR* (int)-IncrementCHIM)/100 ; // Тогда увеличиваем ШИМ на число процентов в относительном выражении
          
          if (ProcChimSR>95) ProcChimSR=95;
        }
        SecTempPrev=Seconds;// Запомним время, когда стабилизировалась температура
      }
    }
   
    
    if (IspReg!=118)// Если не фракционная перегонка, то все как обычно.
    {
      // Если температура в кубе превысила температуру при которой надо отбирать СР
      if (temps[TEMP_KUB]>=tEndRectOtbSR)
      {  
        StateMachine=7;        
        KlOpen[KLP_SR]=0;// Отключаем клапана отбора СР
        KlClose[KLP_SR]=10;
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }
      else    
        break;    
    }
    else
    {
      if (TekFraction<CountFractionRect-1) // смотрим на количество фракций, а оно может измениться при сработке датчика уровня.
      {
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev1) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
           }
          
        }    
       break;        
      }
      else
      {  // Если это последняя фракция, то переводим в режим отбора хвостов.
         SecTempPrev1=Seconds; 
         if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
         StateMachine=7;        
      }
    }
  case 7: // Отбор Хвостов
    UstPower=PowerRect;
    // Подчиненные контроллеры переводим в ректификацию
    digitalWrite(PIN_SLAVE_0, 1);
    digitalWrite(PIN_SLAVE_1, 1);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    
    if (IspReg!=118) // Если не фракционный отбор, то все как обычно
    {
      KlOpen[KLP_SR]=0;
      KlClose[KLP_SR]=10;
      KlOpen[KLP_GLV_HVS]=(timeChimRectOtbGlv/100)*90;
      KlClose[KLP_GLV_HVS]=timeChimRectOtbGlv-KlOpen[KLP_GLV_HVS];
      
      if (temps[TEMP_KUB]>=tEndRect)
      {  
        StateMachine=8;        
        SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }
      else    
        break;
    }
    else
    { // Отбор фракционный, все через клапан отбора СР
      KlOpen[KLP_SR]=(timeChimRectOtbGlv/100)*90;
      KlClose[KLP_SR]=timeChimRectOtbGlv-KlOpen[KLP_SR];

      if (TekFraction<CountFractionRect)
      { // Если количество фракций не превышает заданную. 
        TekTemp=TempFractionRect[TekFraction]; //Текущее состояние - первый недробный перегон
        if (TekTemp>=0) // Значение больше нуля, значит это температура
        {
          if (temps[TEMP_KUB]>=TekTemp)
          {  
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
          }
        }
        else
        {
          // Значение меньше нуля, значит это время в минутах
            SecOstatok=-((Seconds-SecTempPrev1) / 60 + TekTemp);
          // Если времени не осталось, переходим на следующую фракцию  
          if (SecOstatok<=0) 
           { 
             TekFraction++; SecTempPrev1=Seconds; 
             if (TekFraction<CountFractionRect) SetAngle(AngleFractionRect[TekFraction]);
           }
        }    
       break;        
      }
      else
      {
        // Иначе переходим к завершению процесса
        StateMachine=8;        
        SecTempPrev=Seconds;// Запомним время, когда закончилась ректификация
        if (BeepStateProcess) my_beep(BEEP_LONG);
      }     
      
    }
  case 8:// Ждем три минуты для окончания подачи воды
    UstPower=0;
    // Подчиненные контроллеры отключаем
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    
    KlOpen[KLP_GLV_HVS]=0;  // Отключаем клапана отбора хвостов
    KlClose[KLP_GLV_HVS]=10;
    KlOpen[KLP_SR]=0;
    KlClose[KLP_SR]=10;

    if  (Seconds-SecTempPrev>180+ ((int)(FlAvtonom>>1))*60)
    {
      // Переводим автомат в конечное состояние
      StateMachine=100;        
      SecondsEnd=Seconds;
      if (BeepStateProcess) my_beep(BEEP_LONG);
    }
    break;
  case 9:// Ожидание, пока датчик уровня отменяет сработку.
      V3=U_UROVEN; 
      if (V3<UROVEN_ALARM )
      {
        Count_Provodimost++;
        #ifdef TEST
                sprintf(my_tx_buffer,"ALARM VOZVRAT U_UROVEN=%i cnt=%i",U_UROVEN,Count_Provodimost);
                Serial.println(my_tx_buffer);
        #endif      
        // По прошествии минуты устанавливаем флаг сработки.
        if (Count_Provodimost>2) 
        {
          flAlarmUroven=0;
          StateMachine=OldStateMachine;
          Count_Provodimost=0;
        }
      }
      else 
      {
        Count_Provodimost=0;
      }
      break;
 
  case 101: // Превышение температуры в ТСА!!!
  case 102: // Превышение давления
    my_beep(2*BEEP_LONG);
  case 100:// Конечное состояние автомата

    // Отключаем на всякий случай все!
    if (FlToGSM && lastSMSState!=StateMachine) StateToSMS();

    if (StateMachine==100 && BeepEndProcess) my_beep(BEEP_LONG);

    UstPower=0;
    CloseAllKLP();
//    digitalWrite(PIN_REG_ON,LOW);
    digitalWrite(PIN_RZG_ON,!RELAY_HIGH);
    digitalWrite(PIN_ALL_OFF,ALL_OFF_HIGH);
    flAllOff=1;
    digitalWrite(PIN_START_1,!RELAY_HIGH);
    digitalWrite(PIN_START_2,!RELAY_HIGH);
    // Отключаем подчиненные контроллеры
    digitalWrite(PIN_SLAVE_0, 0);
    digitalWrite(PIN_SLAVE_1, 0);
    digitalWrite(PIN_SLAVE_2, 0);
    digitalWrite(PIN_SLAVE_3, 0);

    break;
  }
  #ifdef TESTMEM
    Serial.println("\n[memCheck_Rectif]");
    Serial.println(freeRam());
  #endif
}
